package FestVox::Festival;

$VERSION = '0.01_01';

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter);

use IO::Socket;
use IO::Handle; 
use POSIX;

sub new {
  my $class = shift;
  my @args = @_;

  my $self = {
    'wave_type' => 'nist',
    'audio_play' => 'na_play',
    'host' => 'localhost',
    'port' => '1314',
    'audio_file' => "/tmp/client_tmp$$.nist",
    'file_stuff_key' => "ft_StUfF_key", # defined in speech tools
    'init' => '(voice_kal_diphone)',
  };

  bless $self, $class;

  while (defined(my $k = shift @args)) {
    if (defined(my $v = shift @args)) {
      if (defined $self->{$k}) {
	$self->{$k} = $v;
      } else {
	my $arglist = join ' ', sort keys %$self;
	die "unknown argument '$k' -- expected one of $arglist";
      }
    } else {
      die "odd number of arguments to new()";
    }
  }

  $self;
}

sub start {
  my $self = shift;

  my $handle = IO::Socket::INET->new
      (Proto     => "tcp",
       PeerAddr  => $self->{host},
       PeerPort  => $self->{port})
	  or return 0;

  $handle->autoflush(1);     # so output gets there right away
  print STDERR "[Connected to $self->{host}:$self->{port}]\n";

  # tell the server to send us back a 'file' of the right type
  print $handle "(Parameter.set 'Wavefiletype '$self->{wave_type})\n"
      if defined $self->{wave_type};
  print $handle $self->{init}."\n" 
      if defined $self->{init};

  $self->{handle} = $handle;

  $self->{kidpid} = fork;
  if (not defined $self->{kidpid}) {
    # better to die, but...
    print "Can't fork: $!\n";
    return 0;
  }

  if ($self->{kidpid}) {
    return 1;
  } else {
    $self->child;
    exit; # not reached because child calls exit
  }
}

sub say {
  my $self = shift;
  my @lines = shift;

  while (defined (my $line = shift @lines)) {
    chomp $line;
    $line =~ s/\"/\\\"/g;
    my $f = substr($line, 0, 1);
    $self->scheme("(tts_textall \"$line\" 'file)");
  }
}

sub scheme {
  my $self = shift;
  my $handle = $self->{handle};
  while (defined(my $line = shift @_)) {
    chomp $line;
    print $handle "$line\n\n";
  }
#  kill("TERM", $self->{kidpid}); # send SIGTERM to child
}

sub stop_speaking {
  my $self = shift;
  my $child = $self->{kidpid};

  if (defined $child) {
    # print "INT -> child $child\n";
    kill -2, $child;
  }

  unlink($self->{audio_file}) 
      if -e $self->{audio_file};

  1;
}

sub child {
  setsid; # completely dissasociate from the parent.
  my $self = shift;
  $FestVox::PointyClicky::Instance = undef;

  # we are the child; receive data from the server.
  my ($line, $remains, $result, $sigint);
  my $handle = $self->{handle};

  local $SIG{INT} = sub { 
    if (defined $self->{audio}) {
      print "KILL audio $self->{audio}\n";
      kill -2, $self->{audio};
    }
    $self->{audio} = undef;
  };

  local $SIG{ABRT} = sub { 
    if (defined $self->{audio}) {
      print "KILL audio $self->{audio}\n";
      kill -2, $self->{audio};
    }
    exit 0;
  };

  while (($line = $remains) || (defined ($line = <$handle>))) {
    undef $remains;
    if ($line eq "WV\n") {	# we have a waveform coming
      undef $result;
      if (defined $self->{audio_file}) {
	print "Saving to $self->{audio_file}\n";
	open(AUDIO, ">$self->{audio_file}") 
	    or die "$self->{audio_file}: $!\n";
      } else {
	$self->{audio} = open(AUDIO, "| $self->{audio_play}");
	print "Piping to $self->{audio_play} ($self->{audio})\n";
      }
      while (defined($line = <$handle>) and 
	     ($self->{audio_file} or $self->{audio})) {
	if ($line =~ s/$self->{file_stuff_key}(.*)$//s) {
	  $remains = $1;
	  print AUDIO $line;
	  last;
	}
	print AUDIO $line;
      }
      close AUDIO;

      if (defined $self->{audio_file}) {
	# call the command if we weren't piping in, using a
	# safe pipe open.
	my $c = "$self->{audio_play} $self->{audio_file}";
	$self->{audio} = open ACOM, "| $c";
	close ACOM;
	unlink($self->{audio_file}) if -e $self->{audio_file};
	undef $self->{audio};
      }

      if (defined $self->{audio}) {
	print "Done. ($self->{audio})\n";
	$self->{audio} = undef;
      }

    } elsif ($line eq "LP\n") {
      while ($line = <$handle>) {
	if ($line =~ s/$self->{file_stuff_key}(.*)$//s) {
	  $remains = $1;
	  print STDOUT $line;
	  last;
	}
	print STDOUT $line;
      }
    } else {
      # if we don't recognize it, echo it
      print STDOUT $line;
    }
    $sigint = 0;
  }	 
  exit;
}

sub DESTROY {
  my $self = shift;
  if (defined (my $pid = $self->{kidpid})) {
    if ($pid) {
      print "stopping festival child [$pid]\n";
      kill 'ABRT', $pid if defined $pid;
      select(undef,undef,undef,0.1);
    }
  }
  # $self->SUPER::DESTROY;
}

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

FestVox::Festival -- Control a Festival Server for FestVox

=head1 SYNOPSIS

First, start a festival server.  You may need to set your
access list to allow you to connect.

    # festival --server

Then, in Perl:

    use FestVox::Festival;

    my $festival = new $FestVox::FestVox::Festival;
    $festival->start('(voice_kal_dipone)');
    $festival->say('Hello, Dr. PointyClicky.');

    $festival->stop_speaking;
    $festival->scheme('(voice_us1_diphone)');
    $festival->say("It's nice to see you again.");

=head1 DESCRIPTION

FestVox::Festival is a relatively simple module for 
talking to a Festival server.  

=head1 AUTHOR

Copyright (C) 2000, Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon University.  For use under the same licensing terms
as Perl itself.

=head1 SEE ALSO

 CMU Sphinx   http://www.speech.cs.cmu.edu/sphinx/

 Festival     http://www.cstr.ed.ac.uk/projects/festival/ or
              http://www.speech.cs.cmu.edu/festival/

 FestVox.org  http://www.festvox.org/

 comp.speech  http://www.speech.cs.cmu.edu/comp.speech/

 perl(1), pointyclicky, FestVox::PointyClicky.

=cut

1;

__END__

