package FestVox::InitMap;

=pod

=head1 NAME 

FestVox::InitMap - Initial MethodMapper Map for FestVox::PointyClicky

=head1 SYNOPSIS

    $self->set_map($FestVox::InitMap); 

=head1 DESCRIPTION

FestVox::InitMap is the FestVox::MethodMapper map used to 
instantiate a FestVox::PointyClicky object.  It defines the
parameters and volatiles allowed in PointyClicky.

Use 

    pointyclicky --help

to get a list of the default parameter map for your settings,
or see FestVox/InitMap.pm itself.

=cut

$FestVox::InitMap = {
  ###  parameters are saved between sessions
  'rc_file' => {
    'type'  => 'volatile',
    'doc'   => 'Configuration file',
    'value' => 'default.pcrc',
  },
  'rc_dir'  => {
    'type'  => 'parameter',
    'doc'   => 'Root of the user directory for PointyClicky',
    'value' => "$ENV{HOME}/.FestVox/PointyClicky",
  },
  'script_dirs'  => {
    'type'  => 'volatile',
    'doc'   => 'Array of directories',
    'value' => [ "./Script",
		 "$ENV{HOME}/.FestVox/PointyClicky/Script" ],
  },
  'action_dirs'  => {
    'type'  => 'volatile',
    'doc'   => 'Array of directories',
    'value' => [ "./Action",
		 "$ENV{HOME}/.FestVox/PointyClicky/Action" ],
  },
  'prefix'  => { 
    'type'  => 'parameter', 
    'doc'   => 'The top-level working directory for the collection',
  },
  'prompt_list_file' => { 
    'type'  => 'parameter',
    'doc'   => 'The list of prompts to use',
  },
  'pre_record_command' => {
    'type' => 'parameter',
    'doc' => 'A fixed command to run before starting recording (eg. to set mixer values',
    'value' => 'true', ## 'true' run at the shell is a dummy command.
  },
  'record_command' => {
    'type'  => 'parameter',
    'doc'   => 'Audio record command as a printf() format.' 
	. '  A time flag is magically appended.' ,
    'value' => 'na_record -f 16000 -otype riff -o %s',
  },
  'post_record_command' => {
    'type' => 'parameter',
    'doc' => 'A fixed command to run after recording (eg. to set mixer values back',
    'value' => 'true', ## 'true' run at the shell is a dummy command.
  },
  'time_style' => {
    'type'  => 'parameter',
    'doc'   => 'How recording duration is decided',
    'domain' => 'enum',
    'options' => [qw(track prompt fixed click_stop deadman)],
    'value' => 'prompt',
  },
  'record_time_fixed' => {
    'type'  => 'parameter',
    'doc'   => 'Time to record when using time_style "fixed"',
    'value' => 2,
  },
  'record_time_mult' => {
    'type'  => 'parameter',
    'doc'   => 'Multiplier to apply to the duration of the synthesized prompt to determine how long to record for when using time_style "prompt"',
    'value' => 1.2,
  },
  'play_command' => { 
# Note that older (pre-1.5.6?) releases of ALSA don't have '-f'
#  $param->{audio_record} = 'arecord -f s16b -s 16000 %s'; 
#  $param->{audio_play}   = 'aplay -f s16b -s 16000 %s'; 
    'type'  => 'parameter', 
    'doc'   => 'Audio play command as a printf() format',
    'value' => 'na_play %s',
  },
  'forking' => {
    'type'  => 'parameter', 
    'doc'   => "Are we allowed to fork()?",
    'domain' => 'binary',
    'value' => 1,
  },
  'font_hlist'  => { 
    'type'  => 'parameter', 
    'doc'   => 'Font for the names in the scrolling list',
    'value' => '-*-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-*',
  },
  'hlist_styles'  => { 
    'type'  => 'volatile', 
    'doc'   => 'Text styles for showing status',
  },
  'font_prompt' => {
    'type'  => 'parameter', 
    'doc'   => 'Font for the prompt text box',
    'value' => '-*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*',
  },
  'ask_on_record' => { 
    'type'  => 'parameter', 
    'doc'   => 'Ask when recording a file?',
    'domain' => 'binary',
    'value' => 0,
  },
  'ask_on_record_over' => { 
    'type'  => 'parameter', 
    'doc'   => 'Ask when recording over an existing file?',
    'domain' => 'binary',
    'value' => 1,
  },
  'bgcolor' => { 
    'type'  => 'parameter', 
    'doc'   => 'Default background color',
    'value' => 'lightgray',
  },
  'time_pause' => { 
    'type'  => 'parameter', 
    'doc'   => 'Amount of time to hold() by default',
    'value' => 0.75,
  },
  'iter_default_plan' => { 
    'type'  => 'parameter', 
    'doc'   => 'Plan of Tasks for each Iteration. ";"-seperated.',
    'value' => 'display; play; color("yellow"); hold; color("orange"); record; color;',
  },
  'iter_plan' => { 
    'type'  => 'volatile', 
    'doc'   => 'Current Plan of Tasks. ";"-seperated.',
  },
  'iter_prior' => { 
    'type'  => 'volatile', 
    'doc'   => 'Index of previous Entry in iteration',
  },
  'play_plan' => { 
    'type'  => 'parameter', 
    'doc'   => 'What to look for when play()ing',
    'domain' => 'enum',
    'options' => ['recorded | prompt', 'prompt', 'recorded'],
    'value' => 'recorded | prompt',
  },
  'prompt_wav_dir' => { 
    'type'  => 'parameter', 
    'doc'   => 'The directory (under prefix/) that contains the prompts',
    'value' => 'prompt-wav',
  },
  'prompt_wav_ext' => { 
    'type'  => 'parameter', 
    'doc'   => 'The extension (.suffix) for the prompt waveform files',
    'value' => 'wav',
  },
  'recorded_wav_dir' => { 
    'type'  => 'parameter', 
    'doc'   => 'The directory (under prefix/) for recordings',
    'value' => 'wav',
  },
  'recorded_wav_ext' => { 
    'type'  => 'parameter', 
    'doc'   => 'The extension (.suffix) for the recorded files',
    'value' => 'wav',
  },
  'voice_institution' => {
    'type'  => 'parameter',
    'doc'   => 'Institution at which the work is being done; default is "net"',
    'value' => 'net',
  },
  'voice_domain' => {
    'type'  => 'parameter',
    'doc'   => 'Domain of the material',
    'value' => 'time',
  },
  'voice_speaker' => {
    'type'  => 'parameter',
    'doc'   => 'An identifier for the speaker',
    'value' => $ENV{USER} || 'zzz',
  },

  ######## volatiles are not saved 

  'tk'             => { 'type' => 'volatile', 
			'doc'  => 'Hash of Tk objects',
			'value' => {} },
  'tasks'          => { 'type' => 'volatile', 
			'doc'  => 'Hash map of task names to code references',
			'value' => {} },
  'singleton'      => { 'type' => 'volatile',
			'doc'  => 'Single-Task agenda mechanism' },
  'scripts'        => { 'type' => 'volatile',
			'value' => {} },

  'dirty'          => { 'type' => 'volatile', 'value' => 0 },
  'text_lock'      => { 'type' => 'volatile', 'value' => 1 },
  'iterating'      => { 'type' => 'volatile', 'value' => 0 },
  'setup_finished' => { 'type' => 'volatile', 'value' => 0 },

  'iter_index'     => { 'type' => 'volatile'},
  'recording_file' => { 'type' => 'volatile'},

  'selection'      => { 'type' => 'volatile'},
  'selection_list' => { 'type' => 'volatile'}, 

  'selection_list_index' => {'type' => 'volatile'},

  'festival' => { 'type' => 'volatile' },

};

'Sooth'

__END__


