###########################################################################
##                                                                       ##
##                   Carnegie Mellon University and                      ##
##                   Kevin A. Lenzo and Alan W Black                     ##
##                         Copyright (c) 2000                            ##
##                        All Rights Reserved.                           ##
##                                                                       ##
##  Permission is hereby granted, free of charge, to use and distribute  ##
##  this software and its documentation without restriction, including   ##
##  without limitation the rights to use, copy, modify, merge, publish,  ##
##  distribute, sublicense, and/or sell copies of this work, and to      ##
##  permit persons to whom this work is furnished to do so, subject to   ##
##  the following conditions:                                            ##
##   1. The code must retain the above copyright notice, this list of    ##
##      conditions and the following disclaimer.                         ##
##   2. Any modifications must be clearly marked as such.                ##
##   3. Original authors' names are not deleted.                         ##
##   4. The authors' names are not used to endorse or promote products   ##
##      derived from this software without specific prior written        ##
##      permission.                                                      ##
##                                                                       ##
##  CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK         ##
##  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ##
##  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ##
##  SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE      ##
##  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ##
##  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ##
##  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ##
##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ##
##  THIS SOFTWARE.                                                       ##
##                                                                       ##
###########################################################################
###                                                                      ##
###  Front-end for playing and recording corpora for festvox.            ##
###                                                                      ##
###########################################################################

use strict;
package FestVox::MethodMapper;
use Exporter;
use AutoLoader;
use English;
use Cwd;
use Sys::Hostname;

## support class for MethodMappers 

BEGIN {
  @FestVox::MethodMapper::ISA = qw(Exporter AutoLoader); 
}

sub new {
  my $class = shift;
  my $self  = {};
  bless $self, $class;
  
  $self->set(@_) if @_;

  return $self;
}

sub set_map {
  my $self = shift;
  my $map  = shift;

  for my $k (keys %$map) {
    $self->{$k} = $map->{$k};
  }
  $self;
}

sub get_map {
  my $self = shift;
  my $type  = shift;
  my $map = {};

  for my $var (grep $self->{$_}->{type} eq $type, keys %$self) {
    # bare metal here since it'll be called all the time.
    $map->{$var} = $self->{$var}->{value};
  }
  $map;
}

sub delete_map {
  my $self = shift;
  while (my $k = shift) {
    delete $self->{$k};
  }
  $self;
}

sub get_meta {
  my $self = shift;
  my $what = shift;
  my $method = shift;
  if (defined $self->{$method} 
      and defined $self->{$method}->{$what}) {
    my $it = $self->{$method}->{$what};
    # do something with ARRAY and HASH refs?
    return($it);
  } else {
    undef;
    # warn "$method does't have a meta type $what";
  }
}

sub set_meta {
  my $self = shift;
  my $what = shift;
  my $method = shift;
  my $value = shift;
  if (defined $self->{$method}) {
    $self->{$method}->{$what} = $value;
  } else {
    # warn "$method does't have a meta type $what";
  }
  $self;
}

sub set {
  my $self = shift;

  if (@_) {
    my $class;
    if ($self =~ /^(.*?)=/) {
      $class = $1;
    }
	
    while (my $key = shift @_) {
      my $val = shift @_;
      no strict;
      if (not defined $self->{$key}) {
	my ($p,$f,$l) = caller;
	warn "$class doesn't have a(n) '$key' method [$f line $l]\n"
	    if $class;
      } else {
	# print "SET: $key <$type= $val\n";
	if (defined (my $options = $self->{$key}->{options})) {
	  if (grep { $_ eq $val } @$options) {
	    $self->{$key}->{value} = $val;
	  }  else {
	    if ($self =~ /^(.*?)=/) {
	      my ($p,$f,$l) = caller;
	      my $sane = $options->[0];
	      my $o = join ', ', @$options;
	      warn "${class}->$key: '$val' is not one of ($o). "
		  . "Using '$sane' instead.\n";
	      $self->{$key}->{value} = $sane;
	    }
	  }
	} else {
	  $self->{$key}->{value} = $val;
	}
      }
    }
  }
}

sub get {
  my $self = shift;
  my $method = shift;
  my $caller_file = shift;
  my $caller_line = shift;

  # print "GET: ${self}->$method\n";

  if ($self =~ m/^(.*?)=/) {
    my $class = $1;
    
    no strict ;
    if (not defined $self->{$method}) {
      warn "MethodMapper: $self Can't AutoLoad instance method $method at $caller_file line $caller_line\n";
      return undef;
    } else {
      if (not defined $self->{$method}->{type}) {
	# warn "Unknown method call $method of type $type at $caller_file line $caller_line\n";
	return undef;
      }
      return $self->{$method}->{value};
    }
  } else {
    warn "MethodMapper: Can't invoke $method on $self at $caller_file line $caller_line\n";
    return undef;
  }
}

sub AUTOLOAD {
  my $self = shift ;

  no strict 'vars';

  $method = $AUTOLOAD;
  $method =~ s/^.*:://;

  if (@_) {
    $self->set($method => $_[0]);
  } else {
    my ($p, $file, $line) = caller;
    $self->get($method, $file, $line);
  }
}


sub DESTROY {
  my $self = shift;

  for my $type (keys %$self) {
    for my $param (keys %{$self->{$type}}) {
      undef $self->{$type}->{$param};
    }
  }
  $self->SUPER::DESTROY;
}

sub save_config {
  my $self = shift;
  my $file = shift;

  if (not open STATE, ">$file") {
    warn "MethodMapper: couldn't save state to $file: $!";
    return 0;
  } else {

    my $host = Sys::Hostname::hostname;
    my $username = getpwuid($REAL_USER_ID);

    $self =~ /^(.*?)=/;
    my $class = $1;

    my %copy = %{$self->get_map('parameter')};
    my @params = sort keys %copy;
    
    print STATE "#\n";
    print STATE "# $class Configuration\n";
    print STATE "# Last modified: $username\@$host ".localtime()."\n";
    print STATE "#\n\n";

    for my $k (@params) {
      my $v = '';
      if (not defined $self->{$k}->{value}) {
	$v = '';
      } else {
	$v = $self->{$k}->{value};
      }
      my $t = sprintf "%-20s", $k;
      print STATE "\n";

      print STATE "# $self->{$k}->{doc}\n";
      print STATE "$t => $v\n";
    }
    print STATE "\n";
    close STATE;
  }
  1;
}


sub restore_config {
  my $self = shift;
  my $file = shift;

  if (not open STATE, $file) {
    warn "MethodMapper: couldn't restore state from $file: $!\n";
    return 0;
  } else {
    my $count = 0;
    while (<STATE>) {
      next unless /\S/;
      next if /^\#/;    # comment: FIRST char is a # 
      chomp;
      my ($key, $value) = split /\s+=>\s+/, $_, 2;
      if (defined $key) {
	$self->set($key => $value);
      }
    }
    close STATE;
  }
  1;
}

1;

__END__


=head1 NAME

FestVox::MethodMapper - Abstract Class wrapper for AutoLoader

=head1 SYNOPSIS

  BEGIN {
    @FestVox::MMDerived::ISA = qw(FestVox::MethodMapper 
                                  Exporter AutoLoader); 
  }

  sub new { 
    my $class = shift;
    my @args = @_;

    my $self = FestVox::MethodMapper->new();
    bless $self, $class;

    my $map = {
      'time_style' => {
        'type'  => 'parameter', 
        'doc'   => 'How recording duration is decided',
        'domain' => 'enum',
        'options' => [qw(track prompt fixed click_stop deadman)],
        'value' => 'prompt',
      },

      'iter_plan' => {
        'type'  => 'volatile',
        'doc'   => 'Currently active plan for iteration: perl code.',
        'value' => 'play; color("yellow"); hold(0.75); color("red"); '
                     . 'record; color;' ,  # see FestVox::ScriptLang 

      },
    };

    $self->set_map($map);  
    $self->set(@args) if @args;
    $self;
  }


=head1 DESCRIPTION

FestVox::MethodMapper takes a hash of hashes and creates 
get() and set() methods, with (some) validation, for the
maps listed.  Generally, a C<parameter> is something that
can be saved and restored, whereas a C<volatile> is not
serialized at save-time.

=head1 BUILT-IN METHODS

=over 4

=item new(@args)

Creates and initializes an empty MethodMapper. 
Calls C<set()> with its arguments.

=item set('var' => 'value')

Set the variable C<var> to 
the value C<'value'>.  Checks if C<var> is in the method
map, and complains if it is not.  Enforces enumerated types
of the C<meta> variable C<options> is an array reference.

=item get('var')

Return the value of 'var' if it is defined and in the
method map.

=item var()

C<var> itself is promoted to method status; if given no
argument, it is considered a C<get()>, and if given 
argument(s), it is considered a C<set()>.  Thus, if you
had a parameter called C<active> in the method map, 
MethodMapper would use AutoLoader to create a C<active()>
method (if ever called), so that C<$self->active> would
return the current value, and C<$self->active(1)> would
set it to C<1>.

=item get_meta('type', 'var')

Get the C<meta> data of a given type for a named variable
in th method map.  

  type     e.g. 'volatile', 'parameter'
  doc      some human-readable string do describe this
  value    current value; useful for initialization
  domain   e.g. binary (actually the only type so far)
  options  array reference of allowed values

=item set_meta('type', 'var', value)

Just what you would think.  Sets the C<meta> variable C<type>
of C<var> to C<value>.

=item set_map($map)

Sets the complete map for this object.  See FestVox::InitMap
for a good example of a method map; it is the big one that
FestVox::PointyClicky itself uses.  This should be generalized
to let you set B<which> map, as C<get_map()> below.

=item get_map($type)

Get the map of a particular type, e.g. C<parameter>.  Note
that the object itself is the top-level (complete) map,
since MethodMapper writes into variables in the object
of the same name; the 'map' itself is just the variables
of that C<type>.

=item delete_map(@mapnames)

Delete the mapping for each variable in C<@mapnames>.

=head1 BUGS

Terribly underdocumented.

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

pointyclicky, FestVox::PointyClicky, FestVox::ScriptLang,
FestVox::InitMap, FestVox::Track, FestVox::Singleton, 
FestVox::Task,  FestVox::Festival, FestVox::MethodMapper,
perl(1).

 http://www.festvox.org/
 http://www.cstr.ed.ac.uk/projects/festival.html
 http://www.speech.cs.cmu.edu/festival/

