###########################################################################
##                                                                       ##
##                   Carnegie Mellon University and                      ##
##                   Kevin A. Lenzo and Alan W Black                     ##
##                         Copyright (c) 2000                            ##
##                        All Rights Reserved.                           ##
##                                                                       ##
##  Permission is hereby granted, free of charge, to use and distribute  ##
##  this software and its documentation without restriction, including   ##
##  without limitation the rights to use, copy, modify, merge, publish,  ##
##  distribute, sublicense, and/or sell copies of this work, and to      ##
##  permit persons to whom this work is furnished to do so, subject to   ##
##  the following conditions:                                            ##
##   1. The code must retain the above copyright notice, this list of    ##
##      conditions and the following disclaimer.                         ##
##   2. Any modifications must be clearly marked as such.                ##
##   3. Original authors' names are not deleted.                         ##
##   4. The authors' names are not used to endorse or promote products   ##
##      derived from this software without specific prior written        ##
##      permission.                                                      ##
##                                                                       ##
##  CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK         ##
##  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ##
##  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ##
##  SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE      ##
##  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ##
##  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ##
##  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ##
##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ##
##  THIS SOFTWARE.                                                       ##
##                                                                       ##
###########################################################################
###                                                                      ##
###  Front-end for playing and recording corpora for festvox.            ##
###                                                                      ##
###########################################################################

require 5.004;

package FestVox::PointyClicky;
use strict;

use FestVox;
use FestVox::InitMap;
use FestVox::Singleton;
use FestVox::Track;
use FestVox::ScriptLang;
use FestVox::Festival;

use English;

use Tk 800.000;
use Tk::HList;
use Tk::DirTree;
use Tk::ItemStyle;
use Tk::Text;
use Tk::Dialog;
use Tk qw(exit);

use FileHandle;
use POSIX;
use Cwd;

BEGIN {
  $FestVox::PointyClicky::VERSION = $FestVox::VERSION;
  @FestVox::PointyClicky::ISA = qw(FestVox::MethodMapper 
				   Exporter AutoLoader); 

  $SIG{'INT'} = sub { Tk::exit; };
}

sub new { 
  my $class = shift;
  die "Can't have two instances if FestVox::PointyClicky"
      if defined $FestVox::PointyClicky::Instance;
  
  my $self = FestVox::MethodMapper->new();
  bless $self, $class;
  my @args = @_;

  if (not $ENV{'ESTDIR'} or not $ENV{'FESTVOXDIR'}) {
    print STDERR "\n\n";
    print STDERR " You should set the FESTVOXDIR and ESTDIR variables\n";
    print STDERR " to get the full functionality of FestVox::PointyClicky\n";
    print STDERR " e.g. (under bash):\n\n";
    print STDERR "   export FESTVOXDIR=/usr0/festvox\n";
    print STDERR "   export ESTDIR=/usr0/speech_tools\n";
    print STDERR "\n\n";
  }

  $FestVox::PointyClicky::Instance = $self;

  $self->set_map($FestVox::InitMap);
  $self->prefix(cwd());
  
  $self->set(@args);

  my $rc = $self->rc_file;
  if ($rc and -e $rc) {
    $self->status("Preferences from $rc");
    $self->restore_config($rc);
  } else {
    $rc = 'default_rc' unless $rc;
    my $default = $self->rc_dir . "/$rc";
    if (-e $default) {
      $self->status("Preferences from $default");
      $self->restore_config($default);
    }
  }
  $self->set(@args);

  my $festival = $self->festival;
#  if (not defined $festival) {
#    $festival = FestVox::Festival->new();
#    my $success = $festival->start;
#    if ($success) {
#      $self->festival($festival);
#    } else {
#       $self->status("No festival server at start", 'darkred');
#    }
#  }

  warn "# Record command 'record_command' not set. Will not Record.\n"
      unless $self->record_command;
	  
  warn "# Play command 'play_command' is not set. Will not Play.\n"
      unless $self->play_command;

  $self->singleton
      (FestVox::Singleton->new
       ('callback' => sub { 
	 my $i = $self->iter_index;
	 $self->cback($i); },
	)
       );

  return $self;
}

sub register_script {
  my $self = shift;

  my $name = shift;
  my $file = shift ;

  my $scripts = $self->scripts;
  $scripts->{$name} = $file;
}

sub cback {
  my $self = shift;
  my $index = shift;
  if (not defined $index) {
    return undef;
  }
  my $singleton = $self->singleton;
  return undef unless $singleton;

  $self->status(undef, 'gray');
  $self->update_entry_style($index);
  $self->update_selectbackground;
  $self->update_file_label($index);
}

sub start {
  my $self = shift;

  $self->tk_start;

  if (defined(my $homedir = $self->rc_dir)) {
    $self->install_dlog 
	unless -d $self->rc_dir;
  }

  my @prompts = $self->read_prompts();
  $self->activate_config;
  $self->hlist_populate(@prompts);
  # $self->promptlist_setup;  # dialog is pointless in a new dir
  $self->build_action_menu();
  $self->build_script_menu();
  $self->setup_finished(1);

  my @children = $self->tk->{hlist}->info('children', '');
  if (@children) {
    $self->browse1(0);
  }

  $self->status("This is FestVox::PointyClicky "
    . "v$FestVox::PointyClicky::VERSION", 
		    'darkgreen');

  MainLoop;
}


sub install_dlog {
  my $self = shift;

  my $dialog =
      $self->tk->{mw}->Dialog
	  (-text => "Install in " . $self->rc_dir . "?",
	   -bitmap => 'question', 
	   -title => 'Install',
	   -default_button => 'Install', 
	   -buttons => [qw/Install Cancel/] ); 

  $self->init_rc_dir
      if ($dialog->Show() eq 'Install');
}

sub init_rc_dir {
  my $self = shift;
  my $where = $self->rc_dir;

  if (not -d $where) {
    my @path = split '/', $where;

    my $the_path = '';
    while (@path) {
      my $p = shift @path;
      if ($p) {
	$the_path .= $p;
	
	if (not -d $the_path) {
	  if (not mkdir $the_path, 0777) {
	    $self->status("Can't mkdir($the_path): $!", 'darkred');
	    return 0;
	  }
	  $self->status("mkdir($the_path)", 'darkgreen');
	}
      }
      $the_path .= '/';
    }
  }

  my $actiondir = "$where/Action";
  if (not -d $actiondir) {
    if (not mkdir $actiondir, 0777) {
      $self->status("Can't mkdir($actiondir): $!", 'darkred');
      return 0;
    }
    $self->status("mkdir($actiondir)", 'darkgreen');
    $self->populate_actiondir($actiondir);
  }

  my $scriptdir = "$where/Script";
  if (not -d $scriptdir) {
    if (not mkdir $scriptdir, 0777) {
      $self->status("Can't mkdir($scriptdir): $!", 'darkred');
      return 0;
    }
    $self->status("mkdir($scriptdir)", 'darkgreen');
    $self->populate_scriptdir($scriptdir);
  }

  my $filename = "$where/".$self->rc_file;
  if (not -e $filename) {
    $self->save_config($filename);
  }

  1;
}

sub populate_actiondir {
  my $self = shift;
  my $actiondir = shift;

  if (not opendir DIR, "$actiondir") {
    warn "populate_actiondir: $actiondir: $!";
    return undef;
  }

  $self->action_dirs([$actiondir]);

  my $files =  {
    'Play'       => 'display; play;',
    'Record'     => 'display; record;',
    'Synthesize' => 'display; synth;',

    'pause_play' => 'display; hold; play;',

    'play_pause_record' => 'display; play; color("yellow"); hold; '
	. 'color("orange"); record; color;',

    'pause_record' => 'display; color("yellow"); hold; '
	. 'color("orange"); record; color;',
  };

  for my $f (keys %$files) {
    my $sf = "$actiondir/$f";
    if (not open OUT, ">$sf") {
      warn "can't make $sf: $!\n";
    } else {
      $self->status("Writing $sf", 'blue');
      print OUT "# FestVox::PointyClicky perl action $f\n";
      print OUT "$files->{$f}\n";
      close OUT;
    }
  }
}

sub populate_scriptdir {
  my $self = shift;
  my $scriptdir = shift;

  if (not opendir DIR, "$scriptdir") {
    warn "populate_scriptdir: $scriptdir: $!";
    return undef;
  }

  $self->script_dirs([$scriptdir]);

  my $files =  {
    '0.setup-ldom' => '
my $f = $FestVox::PointyClicky::Instance;
my $inst = $f->voice_institution;
my $dom  = $f->voice_domain;
my $speaker = $f->voice_speaker;
my $ldom_name = join "_", $inst, $dom, $speaker;

# presume upon the name of the prompt list
$f->prompt_list_file("etc/$dom.data");

my $c = "$ENV{FESTVOXDIR}/src/ldom/setup_ldom $inst $dom $speaker";

$f->tk->{mw}->update;
$f->status("Setting up $ldom_name", "darkgreen");
print STDERR "$c\n";
system($c);
if (-e "etc/$dom.data") { 
# update the UI
   $f->hlist_create;
   my @prompts = $f->read_prompts();
   $f->hlist_populate(@prompts);
   $f->activate_config;
} else {
   $f->status("$dom domain init failed", "red");
}

undef $f;
1;
',
    '1.build-prompts' => '
my $f = $FestVox::PointyClicky::Instance;
my $plf = $f->prompt_list_file;
my $c = \'festival -b festvox/build_ldom.scm \' 
                   . \'"(build_prompts \"\'.$plf.\'\")"\';

$f->tk->{mw}->update;
$f->status("Building prompts", "darkgreen");
print STDERR "$c\n";
system($c);

$f->hlist_create;
my @prompts = $f->read_prompts();
$f->hlist_populate(@prompts) ;
$f->activate_config;

$f->status("Ready to record time (ldom).\n");
undef $f;
1;
',

    '9.finalize-ldom' => '
use Cwd;
my $f = $FestVox::PointyClicky::Instance;

my $inst = $f->voice_institution;
my $dom  = $f->voice_domain;
my $speaker = $f->voice_speaker;
my $ldn = join "_", $inst, $dom, $speaker;
my $plf = $f->prompt_list_file;

$f->tk->{mw}->update;
$f->status("Labeling", "darkgreen");
my $c = \'bin/make_labs prompt-wav/*.wav\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Building Utterance Structures", "darkgreen");
$c = \'festival -b festvox/build_ldom.scm "(build_utts \"\'.$plf.\'\")"\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Extracting Pitch Marks", "darkgreen");
$c = \'bin/make_pm_wave wav/*.wav\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Labeling With Pitch Marks", "darkgreen");
$c = \'bin/make_pm_lab pm/*.pm\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Normalizing Power", "darkgreen");
$c = \'bin/simple_powernormalize wav/*.wav\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Deriving mel-cep festures", "darkgreen");
$c = \'bin/make_mcep wav/*.wav\';
print STDERR "$c\n";
system($c);

$f->tk->{mw}->update;
$f->status("Clustering and Building...", "darkgreen");
$c = "festival -b festvox/build_ldom.scm \'(build_clunits \"$plf\")\'"; 
print STDERR "$c\n";
system($c);

my $runit = "festival festvox/${ldn}_ldom.scm \'(voice_${ldn}_ldom)\'";
$f->status("$runit", "darkgreen");

my $dir = cwd;

print STDERR "\n";
print STDERR "You should check the alignments with a tool like emulabel.\n";
print STDERR "\n";
print STDERR "Run your new ldom with something like\n"; 
print STDERR "  \n";
print STDERR "  cd $dir; $runit\n";
print STDERR "  (saytime)\n";
print STDERR "\n";
1;
',
  };

  for my $f (keys %$files) {
    my $sf = "$scriptdir/$f";
    if (not open OUT, ">$sf") {
      warn "can't make $sf: $!\n";
    } else {
      $self->status("Writing $sf", 'blue');
      print OUT "# FestVox::PointyClicky perl script $f\n";
      print OUT "$files->{$f}\n";
      close OUT;
    }
  }
}

sub activate_config {
  my $self = shift;

  my $tk = $self->tk;

  $tk->{text}->configure(-font => $self->font_prompt);
  my $fh = $self->font_hlist;

  for (@{$self->hlist_styles}) {
    $_->configure(-font => $fh);
  }

  $tk->{mw}->idletasks;
}

sub cleanup {
  my $self = shift;
  my $tk = $self->tk;

  if ($self->dirty) {
    my $dialog =
	$tk->{mw}->Dialog
	    (-text => "Save changes?",
	     -bitmap => 'question', 
	     -title => 'Prompts or durations modified', 
	     -default_button => 'Save', 
	     -buttons => [qw/Save Cancel/] ); 
    $self->promptlist_save_dlog()
	  unless ($dialog->Show() ne 'Save');
  }

  my $s = $self->singleton;
  $s->stop if defined $s;

  my $f = $self->festival;
  $f->stop_speaking if defined $f;

  $self->festival(undef);
  undef $f;

  Tk::exit;
}

sub prefix_set {
  my $self = shift;
  my $dir = $self->prefix;
  my ($ok, $cancel) = ('OK', 'Cancel');

  my $tk = $self->tk;

  my $dw = $tk->{mw}->Dialog
      (-title => 'Working directory (prefix)',
       -default_button => $ok,
       -buttons => [$ok, $cancel]
       );
			      
  my $f1 = $dw->Frame;
  $f1->pack(-expand => 1, 
	    -fill => 'both', 
	    -side => 'top');

  my $f2 = $dw->Frame;
  my $ent = $f2->Entry
      (-width => 40,
       -text => $dir,
       );

  my $dirtree = $f1->Scrolled
      ('DirTree',
       -command => sub {
	 $dir = shift;
	 $dir =~ s|^/+|/|;
	 $ent->delete(0, 'end');
	 $ent->insert('end', $dir);
	 $ent->xview('end');
       },
       -directory => $dir,
       )->pack(-expand => 1, 
	       -fill => 'both',
	       -side => 'bottom');

  my $lab = $f2->Label
      (-text => 'Set prefix to',
       -anchor => 'e',
       )->pack(-side => 'left');

  $ent->delete(0, 'end');
  $ent->insert('end', $dir);
  $ent->xview('end');
  $ent->pack(-side => 'left',
		     -expand => 1,
		     -fill => 'x');
  $f2->pack(-side => 'bottom');

  my $button = $dw->Show;
  if ($button eq 'OK') {
    $self->prefix($dir);
    chdir $dir;
    $self->status("prefix = $dir", 'darkgreen');
  }
}

sub selection_set {
  my $self = shift ;
  my @indices = @_;

  return undef unless @indices;

  my $tk = $self->tk;

  for my $i (sort { $a <=> $b } @indices) {
    $tk->{hlist}->selectionSet($i);
  }

  my $index = $indices[0];

  my @children = $tk->{hlist}->info('children', '');

#  if ($self->iterating) {
#    if ($index + 5 < @children) {
#      $tk->{hlist}->see($index + 5);
#    }
#  }
  $tk->{hlist}->see($index);    

#  $tk->{hlist}->anchorSet($index) unless $self->iterating;
  $self->update_file_label($index);
  $self->update_entry_style($index);
  $self->update_selectbackground;
  $self->update_track_time($index);
  $self->update_menus;

  $tk->{mw}->idletasks;

  undef;
}

sub update_hlist {
  my $self = shift;
  my $tk = $self->tk;

  my @toplevel = $self->tk->{hlist}->info('children', '');
  if (0 and @toplevel) {
    for (@toplevel) {
      my @children = $tk->{hlist}->info('children', $_);
      $self->update_entry_style(@children);
    }
  } else {
    $self->update_entry_style(@toplevel);
  }

  return $tk->{hlist};
}

sub add_track {
  my $self = shift;
  my $track = shift;
  my @toplevel;

  if (0) {
    @toplevel = $self->tk->{hlist}->info('children', '');
    if (not @toplevel) {
      my $c = $self->tk->{hlist}->addchild
	  ('',
	   -text => 'root',
	   -itemtype => 'text', 
	   );
      $self->tk->{hlist}->indicator('create', $c);
      @toplevel = $self->tk->{hlist}->info('children', '');
      $self->tk->{mw}->update;
    }
  } else {
    @toplevel = ('');
  }

  my $child = $self->tk->{hlist}->addchild
      ($toplevel[-1],
       -text => $track->basename, 
       -itemtype => 'text', 
       -data => $track); 
}

sub get_track {
  my $self = shift;
  my $index = shift;
  my $track = $self->tk->{hlist}->info('data', $index);

  $track;
}


sub update_track_time {
  my $self = shift; 
  my $index = shift;

  my $tk = $self->tk;
  my $track = $self->get_track($index);
  return undef unless defined $track;

  my $tl = $self->text_lock;

  $self->set_text_lock(0) if $tl; 
  my $rs = $self->time_style;

  $tk->{tt_from_label}->configure(-text => "from $rs");

  my $rt;
  if ($rs eq 'prompt') {
    $rt = $track->guess_duration;
  } elsif ($rs eq 'track') {
    $rt = $track->time_total;
  } elsif ($rs eq 'fixed') {
    $rt = $self->record_duration;
  } else {
    #
  }

  if (not defined $rt) {
    $rt = $track->guess_duration || $self->record_duration;
  }
  $tk->{time_entry}->delete(0, 'end');
  $tk->{time_entry}->insert(0, $rt);

  $self->set_text_lock($tl) if $tl;
}

sub update_file_label {
  my $self = shift;
  my $index = shift;

  my $tk = $self->tk;
  my $track = $self->get_track($index);

  return undef unless defined $track;

  # manage the file label
  my $r = $track->recorded;
  my $p = $track->prompt_wav;
  my $name = $track->basename;

  if ($r) {
    my $s = -s $track->recorded_wav_file;
    $s = $s / 1024;

    if ($p) {
      $tk->{file_label}->configure
	  (-text => 
	   sprintf("$name: recording exists (%0.1fK), prompt exists",$s),
	   -foreground => 'blue');
    } else {
      $tk->{file_label}->configure
	  (-text => 
	   sprintf("$name: recording exists (%0.1fK), no prompt",$s),
	   -foreground => 'blue');
    }
  } elsif ($p) {
    $tk->{file_label}->configure
	(-text => "$name: unrecorded (prompt exists).",
	 -foreground => "darkred");
  } else {
    $tk->{file_label}->configure
	(-text => "$name: no audio (neither prompt nor recorded)",
	 -foreground => 'darkred');
  }

}

sub start_iterating {
  my $self = shift;
  my $plan = shift || $self->iter_default_plan;

  my $tk = $self->tk;

  my $hlist = $tk->{hlist};

  my @selection = 
      sort { $a <=> $b } $hlist->selectionGet;
  return undef unless @selection;

  $self->iterating(1);
  $self->iter_plan($plan);
  $self->iter_prior(undef);

  my $index = shift @selection;

  $tk->{mw}->update;
  while ($self->iterate($index)) {
    $index = shift @selection;
    $tk->{mw}->update;
  }

};

sub iterate {
  my $self = shift;
  my $index = shift;

  my $tk = $self->tk;
  $self->singleton->wait(sub { $tk->{mw}->update; });

  if (not $self->iterating or not defined $index) {
    $self->stop_iterating;
    return undef;
  }

  my $ip = $self->iter_prior;
  $tk->{hlist}->selectionClear($ip) if defined $ip;

  $self->iter_index($index);
  $tk->{hlist}->anchorSet($index);

  my $ccount = scalar($tk->{hlist}->info('children', ''));
#  if ($index + 5 < $ccount) {
#    $tk->{hlist}->see($index + 5);
#  } else {
    $tk->{hlist}->see($index);    
#  }

  my $track = $self->get_track($index);
  if (defined $track) {
    my $name = $track->basename;

    my $st = "$name:";
    $self->status($st);
    $self->update_file_label($index);

    ## here's the action
    $self->do_plan($self->iter_plan);
    ##

    $tk->{i_label}->configure(-text => '',
			      -foreground => 'black');
  }
  $self->iter_prior("$index");

  # intentionally trigger a ghost waiter at the end
  1;
};

sub stop_iterating {
  my $self = shift;
  my $tk = $self->tk;

  if ($self->iterating) {
    $tk->{i_label}->configure
	(-text => '',
	 -foreground => 'black');

    $tk->{i_label}->update;
    $self->status(undef, 'black');
    $self->iterating(0);
    1;
  } else {
    0;
  }
  $self->bgcolor_set(undef);
  $tk->{mw}->update;
}

sub browse1 { 
  my $self = shift;
  my $o = shift;

  if (defined $o) {
    $self->selection_set($o); 
    $self->iter_index($o);
    &display();
  }

  undef;
};

sub command1 {
  my $self = shift;

  $self->key_space;
  $self->singleton->wait(sub { $self->tk->{mw}->update; });
  $self->start_iterating($self->iter_default_plan) unless $self->iterating;
  
  undef;
}

sub key_return {
  my $self = shift;

  $self->key_space;
  $self->singleton->wait(sub { $self->tk->{mw}->update; });
  $self->start_iterating($self->iter_default_plan) unless $self->iterating;
  
  undef;
}

sub key_space {
  my $self = shift;
  my $tk = $self->tk;

  $self->singleton->stop;
  $self->stop_iterating;
  if (defined (my $f = $self->festival)) {
    $f->stop_speaking;
  }
  $self->tk->{mw}->update;

  0;
}

sub button1 {  # Button-1
  my $self = shift;

  $self->singleton->stop;
  $self->stop_iterating;
  $self->tk->{mw}->update;

  undef;
};

sub double1 { 
  my $self = shift;
  $self->stop_iterating;

  my $singleton = $self->singleton;
  if ($singleton->active) {
    $singleton->stop();
  } else {
    $self->start_iterating('play;') unless $self->iterating;
  }
  $self->tk->{mw}->update;
};

sub button3 { 
  my $self = shift;
  $self->stop_iterating;
  $self->singleton->stop();
  $self->tk->{mw}->update;
  undef;
};

sub double3 { 
  my $self = shift;
  $self->stop_iterating;

  my $singleton = $self->singleton;
  if ($singleton->active) {
    $singleton->stop();
  } else {
    $self->start_iterating('record;');
  }
  $self->tk->{mw}->update;
  undef;
};


sub hlist_populate {
  my $self = shift;
  my @corpus = @_;
  return undef unless @corpus;
  return undef if @corpus == 1 and not defined $corpus[0];
  my $tk = $self->tk;
  
  my $c = 0;

  for my $track (@corpus) {
    $self->add_track($track);
  }    

  $tk->{hlist}->update;

  $tk->{hlist}->configure(-browsecmd => sub { $self->browse1(@_) });
  $tk->{hlist}->bind('<Button-1>'    => sub { $self->button1(@_) });
  $tk->{hlist}->bind('<Double-1>'    => sub { $self->double1(@_) });
  $tk->{hlist}->bind('<space>'       => sub { $self->key_space(@_) });

  $tk->{hlist}->bind('<p>'  => sub { $self->start_iterating('play') });
  $tk->{hlist}->bind('<r>'  => sub { $self->start_iterating('record') });
  $tk->{hlist}->bind('<d>'  => sub { $self->delete_selection; });

#  $tk->{hlist}->configure(-command => sub { $self->key_return(@_)});
  $tk->{hlist}->bind('<Return>'      => sub { $self->key_return(@_) });

  $tk->{hlist}->bind('<Button-3>'  => sub { $self->button3(@_)});
  $tk->{hlist}->bind('<Double-3>'  => sub { $self->double3(@_)});

  $self->update_hlist;
}

sub update_entry_style {
  my $self = shift;
  my $which;

  my $tk = $self->tk;

  if (@_) {
    for my $entry (@_) {
      # set the text styles we need to represent the state
      # of each prompt. 

      my $track = $self->get_track($entry);
      next unless defined $track;

      my $sty;
      my $name = $track->basename;

      my $r = $track->recorded   ? 2 : 0;
      my $p = $track->prompt_wav ? 1 : 0;

      $which = $r + $p;
      $sty = $self->hlist_styles->[$which];

      $tk->{hlist}->entryconfigure
	  ($entry,
	   -itemtype => 'text',
	   -style    => $sty,
	   );
    }
  }

  $tk->{hlist}->update;
}

sub update_selectbackground {
  my $self = shift;
  my ($which, $which_show);

  my $tk = $self->tk;

  my @sel = $tk->{hlist}->selectionGet;
  if (@sel) {
    for my $entry (@sel) {
      # set the text styles we need to represent the state
      # of each prompt. 

      my $track = $self->get_track($entry);
      next unless defined $track;

      my $name = $track->basename;

      my $r = $track->recorded   ? 2 : 0;
      my $p = $track->prompt_wav ? 1 : 0;

      $which = $r + $p;
      $which_show = $which unless defined $which_show;
      $which_show = -1 unless $which == $which_show;
      last if $which_show == -1;
    }
  }

  $which_show = -1 unless defined $which_show;

  if ($which_show == -1) {
    $tk->{hlist}->configure(-selectbackground => 'lightgreen')
	unless $self->iterating;
  } else {
    if ($which_show > 1) {
      $tk->{hlist}->configure(-selectbackground => 'cyan');
    } elsif ($which_show) {
      $tk->{hlist}->configure(-selectbackground=> 'yellow');
    } else {
      # nothing
      $tk->{hlist}->configure(-selectbackground=> 'orange');
    }
  }
}

sub hlist_create {
  my $self = shift;
  my $tk = $self->tk;

  # force an update first, so the menus appear while loading 
  # a large file...
  $tk->{mw}->update;

  if ($tk->{hlist}) {
    $tk->{hlist}->delete('all');
    $tk->{hlist}->destroy;
  }

  my $parent_frame = $tk->{frame1};
  $tk->{hlist} = $parent_frame->Scrolled
      ('HList',
#     -header => 1,
       -columns => 1,
       -width => 10,
       -background => $self->bgcolor,
       -height => 20,
       -selectmode => 'extended',
       -scrollbars => 'e',
       -itemtype => 'text',
#       -indicator => 1,
       -selectbackground => 'yellow',
       )->pack
	   (
	    -side => 'left',
	    -anchor => 'nw',
#	    -expand => 1,
	    -fill => 'y',
	    );


  $self->hlist_styles(make_styles($self->font_hlist, $tk->{hlist}));
  $tk->{hlist};

}

sub make_styles {
  my $fh = shift;
  my $hlist = shift;

  my $styles = [];
  $styles->[0] = $hlist->ItemStyle
      ('text',
       -font => $fh,
       -foreground => 'darkred',
       -background => 'gray',
       );
  $styles->[1] = $hlist->ItemStyle
      ('text',
       -font => $fh,
       -foreground => 'darkred',
       -background => 'white',
       );
  $styles->[2] = $hlist->ItemStyle
      ('text',
       -font => $fh,
       -foreground => 'blue',
       -background => 'gray',
       );
  $styles->[3] = $hlist->ItemStyle
      ('text',
       -font => $fh,
       -foreground => 'blue',
       -background => 'white',
       );

  $styles;
}

sub set_text_lock {
  my $self = shift;
  my $state = shift;

  if (not defined $state) {
    $state = $self->text_lock;
  } else {
    $self->text_lock($state);
  }

  my $tk = $self->tk;
  if ($state > 0) {
    $tk->{text}->configure(-state => 'disabled');
    $tk->{text_update_but}->configure(-state => 'disabled');
    $tk->{time_entry}->configure(-state => 'disabled');
  } else {
    $tk->{text}->configure(-state => 'normal');
    $tk->{text_update_but}->configure(-state => 'normal');
    $tk->{time_entry}->configure(-state => 'normal');
  }
}

sub save_prompts {
  my $self = shift;
  my $file = shift;

  if (not open STATE, ">$file") {
    $self->status("couldn't save corpus info to $file: $!\n", "red");
    warn "couldn't save state to $file: $!";
    return 1;
  } else {

    my $host = Sys::Hostname::hostname;
    my $username = getpwuid($REAL_USER_ID);

    print STATE "#\n";
    print STATE "# FestVox::PointyClicky Corpus Information\n";
    print STATE "# Last modified: $username\@$host ".localtime()."\n";
    print STATE "#\n\n";

    my $tk = $self->tk;
    my @children = $tk->{hlist}->info('children', '');
    for my $i (@children) {
      my $track = $self->get_track($i); 
      next unless defined $track;
      my $c = $track->content;
      $c =~ s/\"/\\\"/g;
      $c =~ s/\n/\\n/g;
      printf STATE ("( %s\t\%s\t%d )\n", 
		    $track->basename,
		    $track->content,
		    $track->time_total || 0,
		    );
    }

    close STATE;
    $self->dirty(0);
    $self->status("Wrote $file", 'darkgreen');
  }
}

sub read_prompts {
  my $self = shift;
  my @series;

  # read in the control file of prompts and associated
  # file basenames.  
  #
  my $p = $self->prefix;
  my $gp = $self->guess_prefix;

  if (defined $gp and $gp ne $p) {
    $p = $self->ask_prefix($gp);
  }

  my $prompt_list_file = $self->prompt_list_file;
  return undef unless defined $prompt_list_file and $prompt_list_file;

  $self->status("reading $prompt_list_file", "blue");
  
  $p = "$p/$prompt_list_file";
  $p = $prompt_list_file unless -e $p;

  if (not -e $p) {
    warn "$p does not exist\n";
    return undef;
  } elsif (not -T $p) {
    # warn "$p is not a text file\n";
    return undef;
  }

  open PROMPTS, $p
      or die "can't open prompt file '$p': $!";

  my $line = 0;

  while (<PROMPTS>) {
    next if /^\#/;
    next unless /\S/;
    chomp;
    $line++;
    if (not s/^\s*\(\s*//) {
      warn "** $prompt_list_file is not a readable prompts file\n";
      warn "** '$_'\n";
      undef @series;
      last ;
    }
    s/\s+\)\s*$//;

    my ($file, $prompt, $time, @rest);
    if (s/^\s*(\S+)\s*//) {
      $file = $1;
    }

    if (s|^(\".*\")\s*||) {
      $prompt = $1;
    } elsif (s|(\(.*\))\s*||) {
      $prompt = $1;
    } else {
      $prompt = $_;
    }

    if (/\t/) {
      ($time, @rest) = split /\t/, $_;
    } elsif (/^(\d+)$/) {
      $time = $1;
    }

    if ($file) {
      $prompt =~ s/\\\"/\"/g;
      $prompt =~ s/\\n/\n/g;
      my $track = FestVox::Track->new('top' => $self,
				      'basename' => $file,
				      'content' => $prompt);
      push @series, $track;
    } else {
      warn "** $prompt_list_file is not a readable prompts file\n";
      undef @series;
      last ;
    }
  }
  close PROMPTS;

  $self->dirty(0);

  if (@series) {
    my $t = sprintf "%d prompts read", scalar @series;
    $self->status($t, 'darkgreen');
  } else {
    $self->status("nothing in corpus", 'darkred');
  }

  @series ? @series : ();
}


sub guess_prefix {
  my $self = shift;

  my $guess_prefix = $self->prefix;

  my $plf = $self->prompt_list_file;
  if (not $plf) {
    # warn "no prompt list file name given\n";
    return undef;
  } else {
    $guess_prefix = cwd();

    if (substr($plf, 0, 1) ne '/') {
      # a relative path. make it absolute.
      $plf = "$guess_prefix/$plf";
    }

    if ($plf =~ m|^(.*)/etc/[^/]+|) {
      $guess_prefix = $1;
    } elsif ($plf =~ m|^(.*)/[^/]+|) {
      $guess_prefix = $1;
    }
  }

  $guess_prefix;
}

sub ask_prefix {
  my $self = shift ;
  my $guess_prefix = shift || $self->prefix || '';

  my $tk = $self->tk;
  my $dialog =
      $tk->{mw}->Dialog
	  (-text => "Change PREFIX to\n$guess_prefix?", 
	   -bitmap => 'question', 
	   -title => 'Change PREFIX', 
	   -default_button => 'OK', 
	   -buttons => [qw/OK Cancel/] ); 

  if ($dialog->Show() eq 'OK') {
    $self->prefix($guess_prefix);
  }
  $tk->{mw}->update;
  $self;
}

sub tk_start {
  my $self = shift;

  $self->tk({}) unless defined $self->tk;
  my $tk = $self->tk;

## start building the Perl/Tk objects we need
  $tk->{mw} = MainWindow->new
      (-title => "FestVox::PointyClicky v$FestVox::PointyClicky::VERSION");
  $tk->{mw}->bind('all', '<Control-c>' => $SIG{INT} );
  $tk->{mw}->bind('all', '<space>'  => sub { $self->key_space(@_) });
  $tk->{mw}->bind('all', '<Button-1>', sub { $self->stop_iterating });

  # $tk->{mw}->title("PointyClicky");
  $tk->{mw}->geometry("750x350");

  my $bgcolor = $self->bgcolor;
  
  # frame 1 is the left-hand frame with the HList and Play OM
  $tk->{frame1} = $tk->{mw}->Frame
      (-relief => 'groove',
       -background =>$bgcolor,
       -borderwidth => 2,
       )->pack(-side => 'left',
	       -fill => 'y',
	       -anchor => 'nw',
	       );

  # frame 2 is the right-hand frame
  $tk->{frame2} = $tk->{mw}->Frame
      (-relief => 'groove', 
       -background =>$bgcolor,
       -borderwidth => 2)
	  ->pack(-side => 'left',
		 -expand => 1,
		 -fill => 'both',
		 -anchor => 'nw');

  # frame 4 is the Iteration: option menu and label
  $tk->{frame4} = $tk->{frame2}->Frame
      (-relief => 'flat', 
       -background =>$bgcolor,
       -borderwidth => 0)
	  ->pack(-side => 'top');

  $tk->{i_label} = $tk->{frame2}->Label
      (-text => '',
       -background =>$bgcolor,
       -relief => 'flat')
	  ->pack(-side => 'top');

  # frame 3 is the status label at the bottom and the prompt area
  $tk->{frame3} = $tk->{frame2}->Frame
      (-relief => 'groove', 
       -background =>$bgcolor,
       -borderwidth => 2)
	  ->pack(-side => 'top',
		 -expand => 1,
		 -fill => 'x',
		 );

  $tk->{status_label} = $tk->{frame3}->Label
      (-background =>$bgcolor)->pack
	  (-side => 'bottom', -fill => 'x');

  $tk->{file_label} = $tk->{frame3}->Label
      (-background =>$bgcolor)->pack
	  (-side => 'bottom', -fill => 'x');

  $tk->{framep} = $tk->{frame3}->Frame
      (-background => $bgcolor,
       -width => 300,
       )->pack(-side => 'bottom',
	       -fill => 'x',
#	       -expand => 1,
	       );

  $tk->{frame5} = $tk->{framep}->Frame
      (-background => $bgcolor
       )->pack(-fill => 'x',
	       -expand => 1,
	       );

  $tk->{text_update_but} = $tk->{frame5}->Button
      (-text => "Update",
       -state => 'disabled',
       -command => sub { $self->update_entries },
       )->pack(-side => 'right');

  $tk->{frame5}->pack(-side => 'right',
#		      -fill => 'x',
#		      -expand => 1,
		      );

  $tk->{frame6} = $tk->{framep}->Frame
      (-background => $bgcolor
       )->pack(-side => 'left',
	       -fill => 'x',
	       -expand => 1);

  $tk->{text_lock_ckbox} = $tk->{frame6}->Checkbutton
      (-text => 'Locked',
       -variable => \$self->{text_lock}->{value},
       -background => $bgcolor,
       -command => sub { 
	 $self->set_text_lock() },
       )->pack
	   (-side => 'left');

  $tk->{tt_label} = $tk->{frame5}->Label
      (-text => 'Time:',
       -background => $bgcolor,
       )->pack(-side => 'left');

  $tk->{time_entry} = $tk->{frame5}->Entry
      (-width => 4,
       -state => 'disabled',
       )->pack(-side=>'left');

  $tk->{tt_from_label} = $tk->{frame5}->Label
      (-text => '',
       -background => $bgcolor,
       )->pack(-side => 'left');

  $tk->{time_entry}->bind
      ('<Return>' => sub { 
	my $tt = $tk->{time_entry}->get;
	$tt = undef unless $tt;
	my @sel = $tk->{hlist}->selectionGet;
	my $i = $sel[0];
	my $track = $self->get_track($i);
	next unless defined $track;
	$track->time_total($tt);
	$self->update_track_time($i);
	$self->dirty(1);
      },
       );

  $tk->{text} = $tk->{frame3}->Text
      (-background =>$bgcolor,
       -exportselection => 1,
       );
  $tk->{text}->configure
      (-font => $self->font_prompt,
       -wrap => 'word',
       );

  $tk->{text}->pack(-side => 'right',
		    -expand => 1,
		    -fill => 'both',
		    -anchor => 'nw');

  # Set up menus

  $tk->{menu_bar} = $tk->{mw}->Menu
      (-background =>$bgcolor);
  $tk->{mw}->configure(-menu => $tk->{menu_bar});

  $tk->{file_menu} = $tk->{menu_bar}->cascade
      (-label => '~File', 
       -tearoff => 0);

  $tk->{file_menu}->command
      (-label => "~Open List...",
       -command => sub { $self->promptlist_dlog },
       );

  $tk->{file_menu}->command
      (-label => "~Save List...",    
       -command => sub { $self->promptlist_save_dlog },
       );

  $tk->{file_menu}->separator;

  $tk->{file_menu}->command
      (-label => "Save ~Preferences",    
       -command => sub { 
	 my $path = $self->rc_dir;
	 my $file = $self->rc_file;
	 if ($self->save_config("$path/$file")) {
	   $self->status("Wrote $path/$file", 'darkgreen');
	 } else {
	   $self->status("Couldn't write $path/$file", 'darkgreen');
	 }
       });

  $tk->{file_menu}->command
      (-label => "Save Preferences ~As...",    
       -command => sub { 
	 my $file = $tk->{mw}->getSaveFile
	     (-initialfile => $self->rc_file);
	 if ($file) {
	   $self->save_config($file);
	   $self->status("Wrote $file", 'darkgreen');
	 }
       });

  $tk->{file_menu}->separator;

  $tk->{file_menu}->command
      (-label => "~Load Preferences",    
       -command => sub { 
	 my $path = $self->rc_dir;
	 my $file = $self->rc_file;
	 if ($file) {
	   $self->restore_config("$path/$file");
	   $self->status("Absorbed $path/$file", 'darkgreen');
	   $self->hlist_create;
	   my @prompts = $self->read_prompts();
	   $self->hlist_populate(@prompts);
	   $self->activate_config;
	 }
       });

  $tk->{file_menu}->command
      (-label => "Load Preferences ~From...",    
       -command => sub { 
	 my $file = $tk->{mw}->getOpenFile
	     (-defaultextension => 'view',
#	      -initialfile => $self->rc_file
	      );
	 if ($file) {
	   $self->restore_config($file);
	   $self->status("Absorbed $file", 'darkgreen');
	   $self->hlist_create;
	   my @prompts = $self->read_prompts();
	   $self->hlist_populate(@prompts);
	   $self->activate_config;
	 }
       });

  $tk->{file_menu}->separator;

  $tk->{file_menu}->command
      (-label => "~Initialize home directory",    
       -command => sub { $self->install_dlog },
       );

  $tk->{file_menu}->separator;

  $tk->{file_menu}->command
      (-label => "~Quit",  
       -command => sub { $self->cleanup; });

###

  $tk->{edit_menu} = $tk->{menu_bar}->cascade
      (-label => "~Edit", 
       -background =>$bgcolor,
       -tearoff => 0);

  $tk->{select_menu} = $tk->{edit_menu}->cascade
      (-label => "~Select", 
       -background =>$bgcolor,
       -tearoff => 0);

  $tk->{select_menu}->command
      (-label => "~Remainder",    
       -command => sub { 
	 $tk->{hlist}->selectionClear();
	 my $first;
	 my @children = $tk->{hlist}->info('children', '');
	 for my $i (0..@children -1) {
	   my $x = $self->get_track($children[$i]);
	   if (defined $x and not $x->recorded) {
	     $first = $children[$i] unless defined $first;
	     $tk->{hlist}->selectionSet($children[$i]);
	   }
	 }
	 if (defined $first) {
	   $tk->{hlist}->anchorSet($first);
	   $self->browse1($first);
	 }
       });

  $tk->{select_menu}->command
      (-label => "~Completed",    
       -command => sub { 
	 $tk->{hlist}->selectionClear();
	 my $first;
	 my @children = $tk->{hlist}->info('children', '');
	 for my $i (0..@children -1) {
	   my $x = $self->get_track($children[$i]);
	   if (defined $x and $x->recorded) {
	     $first = $children[$i] unless defined $first;
	     $tk->{hlist}->selectionSet($children[$i]);
	   }
	 }
	 if (defined $first) {
	   $tk->{hlist}->anchorSet($first);
	 }
       });

  $tk->{select_menu}->command
      (-label => "~All",    
       -command => sub { 
	 my @children = $tk->{hlist}->info('children', '');
	 my $count = scalar(@children);
	 $tk->{hlist}->selectionClear;
	 if ($count) {
	   $tk->{hlist}->anchorSet($children[0]);
	   for (@children) {
	     $tk->{hlist}->selectionSet($children[0], $children[-1]);
	   }
	   $self->browse1(0);
	 }
	 $self->update_menus;
       });

    $tk->{select_menu}->command
	(-label => "~None",    
	 -command => sub { 
	   $tk->{hlist}->selectionClear();
	   $tk->{hlist}->configure(-selectbackground => 'yellow');
	 });

  $tk->{edit_menu}->separator;

  $tk->{delete_menu} = $tk->{edit_menu}->command
      (-label => '~Delete Selected', 
       -command => sub { $self->delete_selection },
       );

  $tk->{edit_menu}->separator;

  $tk->{prefix_menu} = $tk->{edit_menu}->command
      (-label => 'Prefi~x', 
       -command => sub { $self->prefix_set }
       );

  $tk->{pref_menu} = $tk->{edit_menu}->command
      (-label => "~Preferences...",
       -command => sub { $self->config_dlog }
       );

  $tk->{menu_bar}->bind('Tk::Menu',
			'<<Iterate>>' => sub {
			  $self->iterate(@_)});

  $self->hlist_create;

  $self;
}

sub build_action_menu {
  my $self = shift;
  my $tk = $self->tk;

  my $base_dir = $INC{"FestVox.pm"};
  $base_dir =~ s/\.pm$//;

#  my $s1 = "$base_dir/Action";
#  my $s2 = $self->rc_dir . '/Action';
#  my @dirs = ($s1, $s2);

  my $d = $self->action_dirs;
  my @dirs = @$d;

  if ($tk->{action_menu}) {
    $tk->{menu_bar}->delete('3');
    undef $tk->{action_menu};
    undef $tk->{dynamic};
  }

  $tk->{action_menu} = $tk->{menu_bar}->cascade
      (-label => '~Action', 
       -tearoff => 0);

  my $p = $self->iter_default_plan;
  $tk->{iter_menu} = $tk->{action_menu}->command
      (-label => '~0 Default (from prefs)',
       -accelerator => '<Return>',
       -command => sub { $self->start_iterating($p) },
       );
  if (@dirs) {
    $tk->{action_menu}->separator;
  }

  my $i = 0;
  while (my $actiondir = shift @dirs) {
    if (-d $actiondir) {
      my @sc;
      if (opendir DIR, $actiondir) {
	while (defined(my $filename = readdir DIR)) {
	  my $sf = "$actiondir/$filename";
	  next unless -T $sf;
	  $self->register_script($filename, $sf);
	  push @sc, [$filename, $sf];
	}
	closedir DIR;
      } else {
	warn "can't open $actiondir: $!\n";
	next;
      }

      for my $s (sort { $a->[0] cmp $b->[0] } @sc) {
	$tk->{dynamic}->[$i++] = $tk->{action_menu}->command
	    (-state => 'disabled',
	     -label => $i < 9 ? '~'.($i+1).' '.$s->[0] : $s->[0],
	     -command => sub { 
	       $self->start_iterating("script('$s->[1]')");
	     });
      }

      $tk->{action_menu}->separator if @dirs;

    } else {
      # warn "$scriptdir is not a good script directory\n";
    }
  }

  $tk->{menu_bar}->update;

}

sub build_script_menu {
  my $self = shift;
  my $tk = $self->tk;

  my $base_dir = $INC{"FestVox.pm"};
  $base_dir =~ s/\.pm$//;

#  my $s1 = "$base_dir/Action";
#  my $s2 = $self->rc_dir . '/Action';
#  my @dirs = ($s1, $s2);

  my $d = $self->script_dirs;
  my @dirs = @$d;

  if ($tk->{script_menu}) {
    $tk->{menu_bar}->delete('4');
    undef $tk->{script_menu};
    undef $tk->{sc_dynamic};
  }

  $tk->{script_menu} = $tk->{menu_bar}->cascade
      (-label => '~Script', 
       -tearoff => 0);

  my $i = 0;
  while (my $scriptdir = shift @dirs) {
    if (-d $scriptdir) {
      my @sc;
      if (opendir DIR, $scriptdir) {
	while (defined(my $filename = readdir DIR)) {
	  my $sf = "$scriptdir/$filename";
	  next unless -T $sf;
	  $self->register_script($filename, $sf);
	  push @sc, [$filename, $sf];
	}
	closedir DIR;
      } else {
	warn "can't open $scriptdir: $!\n";
	next;
      }

      for my $s (sort { $a->[0] cmp $b->[0] } @sc) {
	$tk->{sc_dynamic}->[$i++] = $tk->{script_menu}->command
	    (
	     -label => $i < 9 ? '~'.($i+1).' '.$s->[0] : $s->[0],
	     -command => sub { 
	       $self->do_plan("script('$s->[1]')");
	     });
      }

      $tk->{script_menu}->separator if @dirs;

    } else {
      # warn "$scriptdir is not a good script directory\n";
    }
  }

  $tk->{menu_bar}->update;

}

sub update_menus {
  my $self = shift;

  my $tk = $self->tk;
  my $state = 'active'; # 'disabled';

  if (defined $tk->{hlist}->selectionGet()) {
    $state = 'active';
  }
  
  # my $cur_state = $tk->{play_menu}->cget('-state');

  if (1) { #$cur_state ne $state) {
    if (my $x = $tk->{dynamic}) {
      for (@$x) {
	$_->configure(-state => $state);
      }
    }
#    $tk->{iter_menu}->configure(-state => $state);

    $tk->{menu_bar}->update;
  }

  $self;
}

sub update_entries { 
  my $self = shift;
  my $tk = $self->tk;

  my $tt = $tk->{time_entry}->get || undef;
  my $i = $self->iter_index;
  if (defined $i) {
    my $track = $self->get_track($i);
    return undef unless defined $track;
    $track->time_total($tt);
    print "getting it $tt\n";
    $self->update_track_time($i);
    $self->dirty(1);
  }

}

sub config_dlog {
  my $self = shift;

  my $tk = $self->tk;
  my $dw = $tk->{mw}->Dialog
      (-title => 'Preferences',
       -default_button => 'OK',
       -buttons => ['OK', 'Cancel']
       );

  my %copy = %{$self->get_map('parameter')};
  my @params = sort keys %copy;

  my %domains;

  my $cb_frame = $dw->Frame();
  my (@cb, @mb, @e);

  foreach my $i (@params) {
    if (my $d = $self->get_meta('domain', $i)) {
      if ($d eq 'binary') {
	my $b1 = $cb_frame->Checkbutton
	    (-text     => $i,
	     -width    => 20,
	     -anchor   => 'c',
	     -variable => \$copy{$i},
	     -relief   => 'flat'
	     )->pack(-side => 'left'); 
      } elsif ($d eq 'enum') {
	my $f = $dw->Frame;
	my $lab = $f->Label
	    (-text => $i,
	     -width => 20,
	     -anchor => 'w',
	     )->pack(-side => 'left');
	if (my $o = $self->get_meta('options', $i)) {

	  my @options = @$o;
	  my $mb = $f->Menubutton
	      (
	       -textvariable       => \$copy{$i},
	       -indicatoron        => 1,
	       -relief             => 'raised',
	       -borderwidth        => 2,
	       -highlightthickness => 2,
	       -anchor             => 'c',
	       -direction          => 'flush',
	       );
	  my $menu = $mb->Menu(-tearoff => 0);
	  $mb->configure(-menu => $menu,
			 -anchor => 'nw',
			 );
	  for my $oi (@options) {
	    $mb->command
		(-label => $oi,
		 -command => sub { $copy{$i} = $oi },
		 );
	  }
	  $mb->pack(-side => 'left');
	  push @mb, $f;
	} else {
	  my $f = $dw->Frame;
	  # no options!
	  my $lab = $f->Label
	      (-text => $i,
	       -width => 20,
	       -anchor => 'w',
	       )->pack(-side => 'left');
	  push @e, $f;
	}
      }
    } else {			# not a 'domain'
      my $f = $dw->Frame;
      my $lab = $f->Label
	  (-text => $i,
	   -width => 20,
	   -anchor => 'w',
	   )->pack(-side => 'left');

      my $ent = $f->Entry
	  (-width => 50,
	   -textvariable => \$copy{$i},
	   );
      $ent->pack(-side => 'right'); # ,-expand => 'yes', -fill => 'x');
      push @e, $f;
    }
  }

  $cb_frame->pack(-side => 'bottom');

  for (@mb, @e) {
    $_->pack(-expand => 'yes',
	     -fill => 'x');
  }

  if ($dw->Show eq 'OK') {
    for (@params) {
      $self->set($_ => $copy{$_});
    }
    $self->activate_config;
  }

}


sub delete_selection {
  my $self = shift ;

  my $tk = $self->tk;
  my @s = $tk->{hlist}->selectionGet;
  if (not @s) {
    $self->status("nothing selected to delete", "red");
    return undef;
  }

  my $pl = '';
  if (@s > 1) { 
    $pl = 's';
  }

  my $dialog = $tk->{mw}->Dialog 
      (-text => "Delete Selected Recording$pl?",
       -bitmap => 'question', 
       -title => 'Delete recording', 
       -default_button => 'Delete', 
       -buttons => [qw/Delete Cancel/] 
       );

  return undef if $dialog->Show() ne 'Delete';

  my $count = 0;
  for my $e (@s) { 
    my $track = $self->get_track($e);
    next unless defined $track;
    if ($track->recorded) {
      my $f = $track->recorded_wav_file;
      print "# Deleting $f\n";
      unlink $f or warn "unlink $f: $!";
      $count++;
    } else {
      print "No recorded file for ".$track->basename.", skipping\n";
    }
  }
  $self->update_entry_style(@s);
  $self->update_selectbackground;
  $self->update_file_label($s[0]);

  if ($count) {
    if (@s > $count) {
      $self->status("Deleted $count files, skipped ".(@s - $count));
    } else {
      $self->status("Deleted $count files.");
    }
  }
}

sub promptlist_setup { 
  my $self = shift;
  if (not defined $self->prompt_list_file) {
    $self->promptlist_dlog;
  }
#  $self->build_action_menu();
};

sub promptlist_dlog { 
  my $self = shift;
  my @types = 
      (
       ["All files", '*'],
       ["Likely files", [qw/.scm .list .txt .text/]]
       );

  my $file;
  my $tk = $self->tk;

  if (-d './etc') {
    my $p = 'etc';
    $file = $tk->{mw}->getOpenFile
	(-initialdir => cwd()."/$p");
#	 -filetypes => \@types);
    # workaround for a Tk::getOpenFile bug. looks pretty brittle.
    $file =~ s|^$p/$p|$p| if $file;
  } else {
    $file = $tk->{mw}->getOpenFile();
# old versions of Perl/Tk don't handle array ref here
#	(-filetypes => \@types);
  }

  if ($file) {
    $self->hlist_create;
    $self->prompt_list_file($file);
    my @prompts = $self->read_prompts();
    $self->hlist_populate(@prompts) ;
  }

  $file;
}

sub promptlist_save_dlog { 
  my $self = shift;
  my @types = 
      (
       ["All files", '*'],
       ["Likely files", [qw/.scm .list .txt .text/]]
       );

  my $file;
  my $tk = $self->tk;

  if (-d './etc') {
    my $p = 'etc';
    $file = $tk->{mw}->getSaveFile
	(-initialdir => cwd()."/$p");
#	 -filetypes => \@types);
    # workaround for a Tk::getOpenFile bug 
    $file =~ s|^$p/$p|$p| if $file;
  } else {
    $file = $tk->{mw}->getSaveFile();
#	(-filetypes => \@types);
  }

  if ($file) {
    if (not $self->save_prompts($file)) {
      $self->prompt_list_file($file);
    }
  }

  $file;
}

#########

sub status {
  my $self = shift;
  my $s = shift;

  if (defined $s) {
    if ($s =~ /\S/) {
      print "# $s";
    }
    print "\n";
  }

  my $tk = $self->tk;
  return undef unless $tk;
  return undef unless $tk->{status_label};

  if (defined $s) {
    $tk->{status_label}->configure(-text => $s);
  }
  if (my $c = shift) {
    $tk->{status_label}->configure(-foreground => $c);
  } else {
    $tk->{status_label}->configure(-foreground => 'black');
  }

  $tk->{status_label}->idletasks;

  $self;
}

sub bgcolor_set {
  my $self = shift;
  my $color = shift ; 

  my $tk = $self->tk;
  if ($color) {
    $tk->{status_label}->configure(-background => $color);
    $tk->{frame1}->configure(-background => $color);
    $tk->{frame2}->configure(-background => $color);
    $tk->{frame3}->configure(-background => $color);
    $tk->{frame4}->configure(-background => $color);
    $tk->{file_label}->configure(-background => $color);
    $tk->{i_label}->configure(-background => $color);
  } else {
    my $bgcolor = $self->bgcolor;
    $tk->{status_label}->configure
	(-background => $bgcolor);
    $tk->{frame1}->configure(-background => $bgcolor);
    $tk->{frame2}->configure(-background => $bgcolor);
    $tk->{frame3}->configure(-background => $bgcolor);
    $tk->{frame4}->configure(-background => $bgcolor)
	if (defined $tk->{frame4});
    $tk->{file_label}->configure(-background => $bgcolor);
    $tk->{i_label}->configure(-background => $bgcolor);
  }

}

sub do_plan {
  my $self = shift;
  my $plan = shift || $self->iter_plan;

  # $self->status("  $plan", 'black');

  # Now I reveal my eval plan...
  # @@ WARNING @@ here's an EVAL! 
  eval $plan;
  # @@
}

sub DESTROY {
  my $self = shift;

  my $s = $self->singleton;
  $s->stop if defined $s;
  my $f = $self->festival;
  $f->stop_speaking if defined $f;
  $self->festival(undef);
  undef $f;

  select(undef,undef,undef,0.1);
  $self->SUPER::DESTROY;
}

1;

__END__

=head1 NAME

FestVox::PointyClicky - Perl/Tk-based User Interface to FestVox

=head1 SYNOPSIS

  use FestVox::PointyClicky;

  FestVox::PointyClicky->new()->start;

  # or 

  my $pc = FestVox::PointyClicky->new('forking' => 1);
  $pc->set('time_pause' => 1.2);
  $pc->start;

  # or use the program 'pointyclicky'

=head1 DESCRIPTION

FestVox::PointyClicky is a graphical user interface for 
manipulating collections of files, motivated by the
needs of the festvox project (http://www.festvox.org).

A PointyClicky object manages a list of atoms, which
are used to collect or create any number of types of
data; the names of these atoms appear in a scrolling
list.  Actions are tied to UI events that invoke 
routines or scripts over the selection, and the scripting
language is Perl itself. See FestVox::PointyClicky::ScriptLang
for the Perl routines provided for playing, recording, 
pausing, and so on.

=head1 DEPENDENCIES

=head2 The FestVox Document and Tools

The FestVox document and tools are available from
the festvox project at http://www.festvox.org.  
PointyClicky is a Perl module distributed with
it, but you may have PointyClicky without the 
rest of the document and tools.  They are highly
recommended.

=head2 Festival and the Edinburgh Speech Tools

FestVox originated as a project to build 
tools for voice building for the Festival Speech Synthesis
System (http://www.cstr.ed.ac.uk/projects/festival.html),
and Festival provides a great deal of functionality
to PointyClicky.  You probably want to have Festival
and the Edinburgh Speech Tools installed, though
these are not required.

=head2 Audio

PointyClicky needs external programs to play and record
audio.  Any decent recording and playing programs will
do; for instance 

=item * B<Edinburgh Speech Tools>: C<na_play> and C<na_record>

=item * B<ALSA>: C<aplay> and C<arecord> utilites.
http://www.alsa-project.org.  

=item * B<SOX>: C<sox>, and the aliases C<record> and C<play>.

The audio player and recorder are set in the C<audio_record> and
C<audio_play> options.

=head1 OPTIONS AND VARIABLES

To be announced.

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

perl(1), http://www.festvox.org, FestVox, pointyclicky,
FestVox::PointyClicky::ScriptLang.

=cut

__DATA__

sub if0 {
#  -indicatorcmd => sub { 
  my $entrypath = shift;
  my $event = shift;
  if ($event eq '<Arm>') {
    my @children = $self->tk->{hlist}->info('children', $entrypath);
    if (@children) {
      my $s = $self->tk->{hlist}->info('hidden', $children[0]);
      if ($s) {
	for (@children) {
	  $self->tk->{hlist}->show('entry', $_);
	}
      } else {
	for (@children) {
	  $self->tk->{hlist}->hide('entry', $_);
	}
      }
    }
  }
#  },
}

