###########################################################################
##                                                                       ##
##                   Carnegie Mellon University and                      ##
##                   Kevin A. Lenzo and Alan W Black                     ##
##                         Copyright (c) 2000                            ##
##                        All Rights Reserved.                           ##
##                                                                       ##
##  Permission is hereby granted, free of charge, to use and distribute  ##
##  this software and its documentation without restriction, including   ##
##  without limitation the rights to use, copy, modify, merge, publish,  ##
##  distribute, sublicense, and/or sell copies of this work, and to      ##
##  permit persons to whom this work is furnished to do so, subject to   ##
##  the following conditions:                                            ##
##   1. The code must retain the above copyright notice, this list of    ##
##      conditions and the following disclaimer.                         ##
##   2. Any modifications must be clearly marked as such.                ##
##   3. Original authors' names are not deleted.                         ##
##   4. The authors' names are not used to endorse or promote products   ##
##      derived from this software without specific prior written        ##
##      permission.                                                      ##
##                                                                       ##
##  CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK         ##
##  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ##
##  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ##
##  SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE      ##
##  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ##
##  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ##
##  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ##
##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ##
##  THIS SOFTWARE.                                                       ##
##                                                                       ##
###########################################################################
###                                                                      ##
###  Front-end for playing and recording corpora for festvox.            ##
###                                                                      ##
###########################################################################

use strict;
use English;

package FestVox::Task;
use FestVox::MethodMapper;
# use POSIX qw/setsid/;

## support class for Tasks 

sub REAPER {
  my $waitedpid = CORE::wait;
  # need to reset the handler on SysV
  $SIG{CHLD} = \&REAPER; 

  if (defined %FestVox::Task::Instance) {
    for my $p (%FestVox::Task::Instance) {
      my $i = $FestVox::Task::Instance{$p};
      my $pid = $i->{pid}->{value};
      if (defined $pid and $pid == $waitedpid) {
	$i->cleanup();
	return undef;
      }
    }
  }
  # print "non-singleton CHLD (harmless): $waitedpid\n";
  undef;
}

# o/` reap... your children well... 
#     the parents shell...
#     will make them zom-bieeees o/`
$SIG{CHLD}= \&REAPER;


BEGIN {
  @FestVox::Task::ISA = qw(FestVox::MethodMapper Exporter AutoLoader); 
}

sub new {
  my $class = shift;
  my @args = @_;
  my $self = FestVox::MethodMapper->new();
  bless $self, $class;

  my $map = {
    'command'  => {"type" => 'volatile', 
		   "doc"  => "the command to execute"},
    'context'  => {"type" => 'volatile', 
		   "options" => ['system', 'code'],
		   "doc"  => "context to evaluate command in"},
    'type'     => {'type' => 'volatile', 
		   'doc'  => "this should really just be a 'data' field"},
    'async'    => {'type' => 'volatile', 
		   'doc'  => "whether we're allowed to execute asyncronously"},
    'active'   => {'type' => 'volatile', 
		   'doc'  => "are we active right now?"},
    'pid'      => {'type' => 'volatile', 
		   'doc'  => "Process ID that will execute the command"},
    'callinit' => {'type' => 'volatile', 
		   'doc'  => "Code ref to call when start() is called"},
    'callback' => {'type' => 'volatile', 
		   'doc'  => "Code ref to call when stop() is called"},
    'time'     => {'type' => 'volatile', 
		   'doc'  => "the time start() was called, or 0"},
  };
  $self->set_map($map);

  $self->set
      (async => 1,
       time  => 0,
       pid   => 0,
       active => 0);

  $self->set(@args) if @args;

  $FestVox::Task::Instance{"$self"} = $self;
  $self;
}

sub start {
  my $self = shift;
  my $command = $self->command;
  return undef unless $command;

  $self->{time}->{value} = time();
  if (my $code = $self->callinit) {
    &$code(@_) if defined $code and $code =~ /^CODE/;
  }

  $self->{active}->{value} = 1;

  if (not $self->async) {
    $self->{pid}->{value} = $$;
    system $command;
    $self->stop;

    no strict 'vars';
    warn "** '$command' returns $CHILD_ERROR"
	if $CHILD_ERROR;

    return 0;
  } else {
    my $child_pid;
    die "$!" unless defined($child_pid = fork);

    if ($child_pid) {           
      # the parent. do some bookkeeping and go on.
      $self->{pid}->{value} = $child_pid;
      return $self;
    } else {
      # the child. 
      # setsid;
      exec $command
	  or die "can't exec $command: $!";
      exit ; # not reached
    }
  }
}

sub wait {
  my $self = shift;
  my $code = shift;
  # wait for the singleton to clear.

  my $closure_iterator = 0;
#  my @states = ('-', '/', '|', '\\');
  my @states = ('-', '\\', '|', '/');
#  my @states = (' ', '.', 'o', '0', 'O', '0', 'o', '.');

  my $w = 0;

  while ($self->active) {
    select(undef,undef,undef,0.1);
    my $c = $states[++$w % @states];
    print "$c\cH";
    &$code() if (defined $code and "$code" =~ /^CODE/);
  }
  print " \cH";
}

sub stop {
  my $self = shift;

  # bare metal and try not to move memory for SIGINT
  if ($self->{active}->{value}) {
    my $pid = $self->{pid}->{value};
    if (not defined $pid) {
      return undef;
    }
    if (not $pid) {
      my ($p,$f,$l) = caller;
      warn "asked to kill PID 0 at $f line $l\n";
      return undef;
    }
    if ($self->{async}->{value} and not $pid == $$) {
      # send a signal to the chile
      kill 'INT', $pid; #  if (-e "/proc/$pid");
    } else {
      # print "# done.\n";
    }
  }
  $self->cleanup();
  undef;
}

sub cleanup {
  my $self = shift;

  # bare metal and try not to move memory for SIGINT
  $self->{active}->{value} = 0;
  $self->{pid}->{value} = 0;
  my $code = $self->{callback}->{value};
  &$code(@_) if defined $code and $code =~ /^CODE/;

  undef;
}

sub DESTROY {
  my $self = shift ;

  $self->stop if $self->{active}->{value};
  undef $FestVox::Task{"$self"};

  $self->SUPER::DESTROY;
}

1;

__END__


=head1 NAME

FestVox::Task - Task object for FestVox::PointyClicky

=head1 SYNOPSIS


    use FestVox::Task;
    my $task = FestVox::Task->new
           ('callback' => sub { $self->cback(@_) });

    $singleton->start('command' => 'echo "hello world"'); 

    # wait for the task to finish
    $task->wait($coderef);

    # the above is equivalent to
    while ($task->active) { 
      select(undef,undef,undef, 0.25);
      &$coderef if defined $coderef and "$coderef" =~ /^CODE/;
    }

    # ...or tell it to stop at a given time. sends SIGINT.
    $task->stop();


=head1 DESCRIPTION

FestVox::Task is the basic unit of subprocess control in
FestVox::PointyClicky; it is derived from FestVox::MethodMapper,
as everything is.

=head1 BUGS

Terribly underdocumented.

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

pointyclicky, FestVox::PointyClicky, FestVox::ScriptLang,
FestVox::InitMap, FestVox::Track, FestVox::Singleton, 
FestVox::Task,  FestVox::Festival, FestVox::MethodMapper,
perl(1).

 http://www.festvox.org/
 http://www.cstr.ed.ac.uk/projects/festival.html
 http://www.speech.cs.cmu.edu/festival/

=cut
