###########################################################################
##                                                                       ##
##                   Carnegie Mellon University and                      ##
##                   Kevin A. Lenzo and Alan W Black                     ##
##                         Copyright (c) 2000                            ##
##                        All Rights Reserved.                           ##
##                                                                       ##
##  Permission is hereby granted, free of charge, to use and distribute  ##
##  this software and its documentation without restriction, including   ##
##  without limitation the rights to use, copy, modify, merge, publish,  ##
##  distribute, sublicense, and/or sell copies of this work, and to      ##
##  permit persons to whom this work is furnished to do so, subject to   ##
##  the following conditions:                                            ##
##   1. The code must retain the above copyright notice, this list of    ##
##      conditions and the following disclaimer.                         ##
##   2. Any modifications must be clearly marked as such.                ##
##   3. Original authors' names are not deleted.                         ##
##   4. The authors' names are not used to endorse or promote products   ##
##      derived from this software without specific prior written        ##
##      permission.                                                      ##
##                                                                       ##
##  CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK         ##
##  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ##
##  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ##
##  SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE      ##
##  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ##
##  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ##
##  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ##
##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ##
##  THIS SOFTWARE.                                                       ##
##                                                                       ##
###########################################################################
###                                                                      ##
###  Front-end for playing and recording corpora for festvox.            ##
###                                                                      ##
###########################################################################

use strict;
package FestVox::Track;
use FestVox::MethodMapper;
## support class for tracks 

# icansing.com! beehive conversation

BEGIN {
  @FestVox::Track::ISA = qw(FestVox::MethodMapper Exporter AutoLoader); 
}

sub new {
  my $class = shift ;
  my $self = FestVox::MethodMapper->new();
  bless $self, $class;

  my $map = {
       'basename' => {'type' => 'parameter', 
		      'doc' => "referent, name, or symbol for this track"},

       # display
       'display_type' => {'type' => 'parameter', 
			  'doc' => "unused"},
       'content' => {'type' => 'parameter', 
		     'doc' => "the text of the track"},
       
       # temporal 
       'time_origin' => {'type' => 'parameter', 
			 'doc' => "Time of the origin"},
       'time_total' =>  {'type' => 'parameter', 
			 'doc' => "The duration of the entire track"},
       'time_units' =>  {'type' => 'parameter', 
			 'doc' => "The metric unit for time (seconds)"},

       'time_start' =>  {'type' => 'parameter', 
			 'doc' => "Beginning of the window we care about"},
       'duration' =>    {'type' => 'parameter', 
			 'doc' => "Length in time_units of the target bit"},

       # 'top' is the object that owns me
       'top' =>         {'type' => 'parameter', 
			 'doc' => "The top level object that owns me. This is a hack for inheriting the prefix and basename from the parent without replicating it everywhere. Might want to changed this, though, because it is so coupled."},
     };
  $self->set_map($map);
  $self->set(@_);

  $self;
};

sub recorded {
  my $self = shift ;
  my $f = $self->recorded_wav_file;
  return $f if -e $f;
  return 0;
}

sub prompt_wav {
  # return the file name if it exists, otherwise undef
  my $self = shift ;
  my $f = $self->prompt_wav_file;
  return $f if -e $f;
  undef;
}

sub recorded_wav_file {
  my $self = shift;
  my $top = $self->top;

  my $name = '';

  my $prefix = $top->get('prefix');
  $name  = "$prefix/" if defined $prefix;

  my $rwd = $top->get('recorded_wav_dir');
  $name .= "$rwd/" if defined $rwd;

  my $bn = $self->basename;
  $name .= $bn if defined $bn;
  
  my $rwe = $top->get('recorded_wav_ext');
  $name .= ".$rwe" if defined $rwe;

  $name;
}

sub guess_duration {
  my $self = shift;
  my $f = $self->prompt_wav;
  my $top = $self->top;
  my $mult = 1.0;
  if ($top) {
    $mult = $top->record_time_mult;
    warn "record_time_mult is 0!\n" unless $mult;
  }

  if (defined $f) {
    my $a = `ch_wave -info $f`;
    if ($a =~ /uration:\s+(\d+\.?\d*)/) {
      my $d = $1;
      $d = sprintf("%0.1f", $mult * $d);
      return $d;
    }
  }
  0;
}

sub prompt_wav_file {
  # the file name, whether it exists or not
  my $self = shift;
  my $top = $self->top;

  my $name = '';

  my $prefix = $top->get('prefix');
  $name  = "$prefix/" if defined $prefix;

  my $pwd = $top->get('prompt_wav_dir');
  $name .= "$pwd/" if defined $pwd;

  my $bn = $self->basename;
  $name .= $bn if defined $bn;
  
  my $pwe = $top->get('prompt_wav_ext');
  $name .= ".$pwe" if defined $pwe;

  $name;
}

1;

__END__


=head1 NAME

FestVox::Track - Track support class for FestVox::PointyClicky

=head1 SYNOPSIS

There is no documentation for this yet.

=head1 DESCRIPTION

Meep.

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

pointyclicky, FestVox::PointyClicky, FestVox::ScriptLang,
FestVox::InitMap, FestVox::Track, FestVox::Singleton, 
FestVox::Task,  FestVox::Festival, FestVox::MethodMapper,
perl(1).

 http://www.festvox.org/
 http://www.cstr.ed.ac.uk/projects/festival.html
 http://www.speech.cs.cmu.edu/festival/

=cut


